#include <string.h>
#include <stdint.h>
#include "GlobalDef.h"

#ifdef PROC_32BIT
  volatile uint32_t CommunFlags;
#endif

// prijem
byte *paketRecBuffer;			// ukazatel na 'buffer', do ktereho se bude prijimat
byte *paketRecBufferAtcPos;		// pro posun v prijmacim bufferu pri prjimani

int paketRecLength = 0;         // delka prijateho paketu (pokud je potreba)
#ifdef PAKET_ADDRESS
  int paketRecAddress = 0;      // adresa obsazena v paketu (0 - neni)
#endif
char paketRecType = PaketTypeNone;  // typ paketu
char paketRecName = PaketNameNone;  // nazev paketu
#ifdef PAKET_IDENT
  int paketRecIdent = 0;        // ident paketu, pokud je pritomen (0 - neni)
#endif

byte *paketRecPosWorkStart = NULL;
byte *paketRecPosWork = NULL;   // aktualni pozice pro zpracovavani (jiz) prijateho paketu

byte *paketSendBuffer = NULL;
int paketSendPos = 0;
int paketSendLength = 0;

// --- INIT (inicializace) ---

void CommunInit(byte *bufferIn, byte *bufferOut)
{
    paketRecBuffer = bufferIn;
}

// --- RECEIVE (prijem) ---

int PaketRecvGetNumberActPos(void);

// test na prijem paketu v predanem poli hodnot
void PaketRecvTestBufer(byte *bufferData, int bufferLength)
{
    // pokud neni predchozi paket zpracovan, tak navrat
    if(CommunFlags & CommunRecDone) { return; }

    // projiti vsech prijatych bytes
    while(bufferLength > 0)
    {
        //char zzz = *bufferData; // test

        // zpracovani tohoto prijateho byte
        PaketRecvTestByte(*bufferData);

        // pokud byl prijat cely paket, tak zde jiz nic
        if (CommunFlags & CommunRecDone) { return; }

        // !!! nevim zda toto nema byt za 'PaketReceiveTestByte'
        bufferData++;      // posun v bufferu s prijatymi daty
        bufferLength--;    // jedna / dalsi hodnota z tohoto bufferu byla zpracovana
    }
}

// prijem / zpracovani pouze jednoho 'byte'
void PaketRecvTestByte(byte value)
{
    // pokud neni predchozi paket zpracovan, tak navrat
    if(CommunFlags & CommunRecDone) { return; }

    // pokud je detekovan zacatek paketu
    if(value == PAKET_START)
    {
        // tak probiha prijem paketu
        CommunFlags |= CommunRecRun;
        // buffer pro prijimany se plni od zacatku
        paketRecBufferAtcPos = paketRecBuffer;
    }

    // pokud se paket aktualne prijima
    if (CommunFlags & CommunRecRun)
    {
        // ulozeni aktualne prijate hodnoty
        *paketRecBufferAtcPos++ = value;

        // test na konec pakeu
        if(value == PAKET_END)
        {
            // STOP znak paketu je jiz ulozen
            // na konec paketu se vlozi 0x00 (pro mozne pouziti)
            *paketRecBufferAtcPos = 0x00;
            // zapise se delka prijateho paketu (pocet hodnot)
            paketRecLength = paketRecBufferAtcPos - paketRecBuffer;

            // paket se jiz neprijima
            CommunFlags &= ~CommunRecRun;

            // vychozi pozice pro zpracovani paketu
            paketRecPosWork = paketRecBuffer + 1;
            #ifdef PAKET_ADDRESS
              // nalezeni adresy v paketu (prvni byte je PAKET_START)
              paketRecAddress = PaketRecvGetNumberActPos();
            #endif
            // vyzvednuti typu paketu
            paketRecName = *paketRecPosWork; paketRecPosWork++;
            #ifdef PAKET_IDENT
              // vyzvednuti identu paketu, pokud je
              paketRecIdent = PaketRecvGetNumberActPos();
            #endif

            // od teto pozice se bude paket prochazet pri zpracovani
            // (aby se zbytecne neprochazel vzdy uplne od zacatku)
            paketRecPosWorkStart = paketRecPosWork;

            // je prijat cely paket a pripraven pro zpracovani
            CommunFlags |= CommunRecDone;
            // (pokracujde ze vyzvednutim dalsiho prijateho znaku)
            // (ten se ale zahodi, pokud neni predchozi paket zpracovan)
        }
    }
}

byte cmdValue = 0;  // cislo prijateho povelu
byte cmdStatus = PaketStatusNone; // 'Status' o vykonani povelu

byte *paketSendBufferActual = NULL; // odkaz na aktualne pouzivany buffer pro vysilany paket

byte PaketProcess(byte* paketOutBuffer)
{
    // zpracovani prijateho paketu

    cmdStatus = 0;
    // ziskani cisla povelu / dat
    cmdValue = PaketRecvGetDataNumber('V');

    // tvorba odpovedi
    // ukazatel na zacatek plneni bufferu
    paketSendBufferActual = paketOutBuffer;
    // zacatek paketu (typ,  nazev)
    PaketSendBeginTypeName(&paketSendBufferActual, PaketTypeAns, paketRecName);
    // odpoved na povel
    PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'V', cmdValue);

    // 'Cmd' paket
    if(paketRecName == PaketNameCmd)
        { MyComPaketCmd(); }

    // 'Data' paket (neni upresneni)
    if(paketRecName == PaketNameData)
        { MyComPaketData(); }

    // pokud je povolena, tak se vysle odpoved na vykonany povel
    if (CommunFlags & CommunSendAns)
    {
        // status vykonani povelu
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'S', cmdStatus);
        // ukonceni paketu
        int paketSendLength = PaketSendEnd(paketOutBuffer, &paketSendBufferActual);
        // odeslani paketu typu ANS na zpracovany povel
        MyComPaketSend(paketOutBuffer, paketSendLength);
    }

    // prijaty paket zpracovan / vykonan / proveden
    PaketRecvProcessed();
    return 1;
}

// --- ---

// test na konec paketu
//int IsPaketEnd(byte value)
//{
//    return ((*paketRecPosWork == PAKET_END) || (*paketRecPosWork == 0));
//}

int PaketRecvGetNumberActPos(void);

// nalezeni zacatku (pozice prvniho znaku) pozadovanych dat
byte* PaketRecvGetDataNamePos(char dataName)
{
	// se paket prochazi vzdy od zacatku (prvni byte PAKET_START)
	paketRecPosWork = paketRecPosWorkStart;

	LabelHledejDale:

    // hleda se v paketu oddelovac dat, tedy ' '
    while (*paketRecPosWork != GROUP_SEPARATOR)
    {
        // konec paketu -> pozadovana hodnota nenalezena (neni pozice / adresa dat v paketu)
        if((*paketRecPosWork == PAKET_END) || (*paketRecPosWork == PAKET_END_TOTAL)) { return NULL; }
        // posun v paketu na dalsi pozici
        paketRecPosWork++;
    }

	// posun na znak / typ dat
	paketRecPosWork++;
	// pokud to neni pozadovany typ dat a za nim rovna se, tedy 'X=', tak se hleda dale
	if ((*(paketRecPosWork) != dataName) || (*(paketRecPosWork + 1) != GROUP_START)) { goto LabelHledejDale; }
	// data nalezena -> posun na zacatek prvni hodnoty dat (znak prvnho cisla, za rovna se)
	paketRecPosWork += 2;

	// vrati se pozice nalezenych dat
	return paketRecPosWork;
}

// vyzvednuti prvni / jedne hodnoty za zadanych dat
int PaketRecvGetDataNumber(char dataName)
{
	// pokud je zadan nazev hodnoty, tak jde o prvni hodnotu
	if(dataName != 0x00)
	{
		// se vzdy postupuje od zacatku paketu
		// pokud nazev dat nenalezen, tak konec / navrat
		if(PaketRecvGetDataNamePos(dataName) == NULL) { return -1; }
	}

	// A) pri hledani prvniho je zde pozice znaku prvniho cisla
	// B) pri hledani dalsiho je zde carka za predchozim cislem

	// pokud je nyni oodelovac polozek, tak se musi preskocit
	while (*paketRecPosWork == ITEMS_SEPARATOR) { paketRecPosWork++; }

	// pokud je jiz konec (skupiny) dat, nebo konec paketu (PAKET_END), nebo 0x00 (konec paketu), tak dalsi hodnota neni
	if ((*paketRecPosWork == GROUP_SEPARATOR) || (*paketRecPosWork == PAKET_END) || (*paketRecPosWork == PAKET_END_TOTAL)) { return -1; }

	// vraceni vyzvednuteho cisla od aktualni pozice
	return PaketRecvGetNumberActPos();
}

// vyzvednuti dalsi hodnoty ze jiz nastavenych dat
int PaketRecvGetDataNumberNext()
{
    // neni zadan nazev hodnoty, pozaduje se dalsi hodnota v poradi
    return PaketRecvGetDataNumber(0x00);
}

int PaketRectHasDataNumberNext()
{
    // je oddelovac polozek || je cislo / hodnota -> je polozka
    if ((*paketRecPosWork == ITEMS_SEPARATOR) && (*(paketRecPosWork + 1) >= '0') && (*(paketRecPosWork + 1) <= '9')) { return 1; }
    return 0;
}

// vyzvednuti cisla z aktualni pozice v paketu
int PaketRecvGetNumberActPos(void)
{
    int number = 0;
    // vycteni cisla, pokud je
    while ((*paketRecPosWork >= '0') && (*paketRecPosWork <= '9'))
    {
        // bude dalsi cislice, tedy posun saoucasneho stavu do vyssiho radu
        number = number * 10;
        // pricteni nove cislice
        number = number + (*paketRecPosWork - '0');
        // posun pozice v bufferu
        paketRecPosWork++;
    }
    return number;
}

// zda aktualne nastavena data (textova) obsahuji pozadovany znak
byte IsCharInActualData(byte *paketPos, char oneChar)
{
	// projiti dat z paketu (dokud neni konec dat (' '), nebo konec paketu (0x00)
	while ((*paketPos != GROUP_SEPARATOR) && (*paketPos != PAKET_END) && (*paketPos != PAKET_END_TOTAL))
	{
		// pokud data obsahuji pozadovany 'znak', tak se vrati '1'
		if (*paketPos == oneChar) { return (byte)1; }
		// posun na dalsi znak v paketu
		paketPos++;
	}
	// doslo se na konec, data neobsahuji pozadovany 'znak', tak se vrati '0'
	return (byte)0;
}

// paket byl jiz zpracovan
void PaketRecvProcessed() { CommunFlags &= ~CommunRecDone; }

// --- Send (vysilani) ---

void itoa(int n, char s[]);
void reverse(char s[]);

// vytvoreni zacatku paketu
void PaketSendBeginName(byte **paketPos, char paketName)
{
	PaketSendBeginTypeName(paketPos, 0x00, paketName);
}

// vytvoreni zacatku paketu s typem a nazvem
void PaketSendBeginTypeName(byte **paketPos, char paketType, char paketName)
{
    byte *ptr = *paketPos;					// snizeni o jeden ukazatel pro snazsi manipulaci
	*ptr++ = PAKET_START;					// pocatecni znak paketu

    // pokud je zdan typ paketu, tak se pouzije
    if(paketType != 0x00)
    {
        *ptr++ = paketType;						// typ paketu
    }

	// pokud ma byt v paketu pritomna adresa, tak se tam vlozi
	#ifdef PAKET_ADDRESS
	  itoa(paketAddressMy, (char*)ptr /* , 10 */);	// prevod hodnoty na retezec
	  while(*ptr != 0) { ptr++; }			// nalezeni konce tohoto retezce s hodnotou (konce teto zapsane hodnoty)
	#endif

	*ptr++ = paketName;						// nazev paketu

	#ifdef PAKET_IDENT
      itoa(paketRecIdent, (char*)ptr /*, 10 */);  // prevod hodnoty na retezec
	  while(*ptr != 0) { ptr++; }				// nalezeni konce tohoto retezce s hodnotou (konce teto zapsane hodnoty)
	#endif

	//*ptr++ = ' ';							// za typem paketu je vzdy mezera
	(*paketPos) = ptr;						// aktualizace ukazatele druhe urovne
}

// vytvoreni konce paketu a vypocet jeho delky
int PaketSendEnd(byte *paketPosStart, byte **paketPos)
{
    byte *ptr = *paketPos;          // pro snizeni poctu ukazatelu
    *ptr++ = PAKET_END;             // koncovy znak paketu
    *ptr = 0x00;                    // konec textoveho retezce (pro jistotu)
    (*paketPos) = ptr;              // aktualizace puvodniho ukazatele
    // vypocet poctu hodnot / bytes v paketu (0x00 jiz neni do paketu)
    return (*paketPos - paketPosStart);
}

//zda jiz bylo zapsano, pro zapis carky pred dalsim cislem
uint8_t numberBefore = 0;

// pokud je 'dataCount' zaporny -> cislo se zapisuje pozadu
void PaketSendAddDataNameAndNumberHexPtr(byte **paketPos, char dataName, uint8_t *dataValuePtr, int dataCount)
{
    // pokud je zadan nazev, tak se zapise do paketu
    if(dataName != 0x00)
    {
        PaketSendAddDataName(paketPos, dataName);
    }

    byte *ptr = *paketPos; 	// snizeni o jeden ukazatel pro snazsi manipulaci
	// pokud jiz bylo cislo, tak se zapise'carka' (jako oodelovac dalsi hodnoty)
	//if(numberBefore == 1) { *ptr++ = ITEMS_SEPARATOR; }

	uint8_t pos = 0; uint8_t rev = 0;
	// pokud je 'dataCount' zaporny -> cislo se zapisuje pozadu
	if (dataCount < 0)
        { dataCount *= -1; pos = (dataCount - 1); rev = 1; }

    uint8_t dataValue;
    // vlozeni potrebnych 'bytes' do paketu
    while(dataCount > 0)
    {
        // precteni jednoho byte a jeho vlozeni do paketu
        // horni 4 bity
        dataValue = (uint8_t)((dataValuePtr[pos] >> 4) & 0x0F); // bity 7-4
        // prevod na Ascii-Hex a vlozeni do paketu
        *ptr++ = ByteToAsciiHex(dataValue);
        // dolni 4 bity
        dataValue = (uint8_t)(dataValuePtr[pos] & 0x0F); // bity 3-0
        // prevod na Ascii-Hex a vlozeni do paketu
        *ptr++ = ByteToAsciiHex(dataValue);

        // jedna hodnota zapsana
        dataCount--;
        // posun na dalsi / predchozi pozici
        if (rev == 0) { pos++; } else { pos--; }
    }

	(*paketPos) = ptr;						// aktualizace ukazatele druhe urovne
	numberBefore = 1;                       // bylo zapsano cislo
}

// vlozeni jedne pojmenovane hodnoty do paketu
void PaketSendAddDataNameAndNumber(byte **paketPos, char dataName, int dataValue)
{
    // pokud je zadan nazev, tak se zapise do paketu
    if(dataName != 0x00)
    {
        PaketSendAddDataName(paketPos, dataName);
    }

	byte *ptr = *paketPos; 	// snizeni o jeden ukazatel pro snazsi manipulaci
	// pokud jiz bylo cislo, tak se zapise'carka' (jako oodelovac dalsi hodnoty)
	if(numberBefore == 1) { *ptr++ = ITEMS_SEPARATOR; }
	itoa(dataValue, (char*)ptr /*, 10 */);  // prevod hodnoty na retezec
	while(*ptr != 0) { ptr++; }				// nalezeni konce tohoto retezce s hodnotou (konce teto zapsane hodnoty)
	(*paketPos) = ptr;						// aktualizace ukazatele druhe urovne
	numberBefore = 1;                       // bylo zapsano cislo
}

// vlozeni pojmenovani pro pozdejsi hodnotu / hodnoty do paketu
void PaketSendAddDataName(byte **paketPos, char dataName)
{
    // snizeni o jeden ukazatel pro snazsi manipulaci
    byte *ptr = *paketPos;
    // zapise se: 'mezera', 'nazev hodnoty', '='
    *ptr++ = GROUP_SEPARATOR; *ptr++ = dataName; *ptr++ = GROUP_START;
    // aktualizace ukazatele druhe urovne
    (*paketPos) = ptr;
    // za nazvem zatim neni zadne cislo
    numberBefore = 0;
}

// vlozeni dalsi hodnoty do paketu (za nastavenou pojmenovanou)
void PaketSendAddDataNumber(byte **paketPos, int dataValue)
{
    // neni nazev hodnoty, jde o dalsi hodnotu z poradi
    PaketSendAddDataNameAndNumber(paketPos, 0x00, dataValue);
}

// vlozeni nekolika hodnot s jednim pojmenovanim do paketu
// dataName -> pojmenovani dat
// (valueType -> Value8Bit / Value16Bit | LOWfirstHIGHsecond / HIGHfirstLOWsecond)
// dataCount -> pocet hodnot
// dataValues -> odkaz na pole hodnot (int[])
void PaketSendAddDataNameAndNumbers(byte **paketPos, char dataName, /* XValuesInBytes valueType, */ int dataCount, int* dataValues)
{
    // index pro zapis dat z dodaneho pole
    int index = 0;
    // zapis nazvu dat a prvni hodnoty
    PaketSendAddDataNameAndNumber(paketPos, dataName, dataValues[index++]);
    // zapis vsech ostatnich dat
    while(index < dataCount)
        { PaketSendAddDataNumber(paketPos, dataValues[index++]); }
}

// vlozeni jednoho pojmenovaneho retezce
void PaketSendAddDataNameAndString(byte **paketPos, char dataName, const char* dataValues)
{
    // pokud je zadan nazev, tak se zapise do paketu
    if(dataName != 0x00)
    {
        PaketSendAddDataName(paketPos, dataName);
    }

	byte *ptr = *paketPos; 	// snizeni o jeden ukazatel pro snazsi manipulaci
	// pokud jiz bylo cislo, tak se zapise'carka' (jako oodelovac dalsi hodnoty)
	if(numberBefore == 1) { *ptr++ = ITEMS_SEPARATOR; }

    // vlozeni retezce do paketu
    // prochazeni retezce az do jeho konce (na konci je 0x00)
    while(*dataValues != 0)
    {
        *ptr = *dataValues;     // kopie hodnoty do paketu
        ptr++; dataValues++;    // posun ukazatelu
    }

	(*paketPos) = ptr;						// aktualizace ukazatele druhe urovne
	numberBefore = 1;                       // bylo zapsano cislo
}

// sestaveni celeho jednoducheho paketu obsahujiciho jednu hodnotu
int PaketSendCreateSimple(byte *paketBuffer, char paketType, char paketName, char dataName, int dataValue)
{
    // pockej na dokonceni aktualniho / predchoziho vysilani
	while(CommunFlags & CommunSendRun);
	// ukazatel na zacatek paketu pro jeho plneni / tvoreni
	byte *paketBufferStart = paketBuffer;
	// vytvoreni zacatku paketu
	PaketSendBeginTypeName(&paketBufferStart, paketType, paketName);
    // vlozeni a pojemnovani a hodnoty pro data
	PaketSendAddDataNameAndNumber(&paketBufferStart, dataName, dataValue);
    // vlozeni konce paketu a vypocet celkove delky
    return PaketSendEnd(paketBuffer, &paketBufferStart);
	//int paketLength = PaketSendEnd(paketBuffer, &paketBufferStart);
	// odeslani paketu (tohoto paketu, takoveto delky)
	//PaketSendToUSB(paketBuffer, paketLength);
    // cekani na skutecne odvysilani paketu (pouze v pripade potreby)
	//while(USARTFlags & USARTSendRun);
}

// pomocne

/* reverse:  reverse string s in place */
void reverse(char s[])
{
    int c, i, j;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

/* itoa:  convert n to characters in s */
void itoa(int n, char s[])
{
    int i, sign;

    if ((sign = n) < 0)  /* record sign */
        n = -n;          /* make n positive */
    i = 0;
    do {       /* generate digits in reverse order */
        s[i++] = n % 10 + '0';   /* get next digit */
    } while ((n /= 10) > 0);     /* delete it */
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
}

// skutecne vyslani podle typu komunikace

// --- USART ---

#ifdef COMMUN_USART
  // zacatek vysilani pres USART
  void PaketSendUSART(UART_HandleTypeDef *huart, byte *bufferData, int bufferLength)
{
    while(CommunFlags & CommunSendRun);	// pockej na konec predesleho vysilani
	CommunFlags |= CommunSendRun;		// paket (ten novy) se jiz prave vysila

	paketSendPos = 0;                   // zacatek vysilani paketu
	paketSendBuffer = bufferData;       // tento paket se vysila
	paketSendLength = bufferLength;     // toto je jeho delka

    PaketSendUSARTNextByte(huart);      // prvni 'byte' do vysilani
    // povoleni preruseni od pradneho vysilaciho registru
    __HAL_UART_ENABLE_IT(huart, UART_IT_TXE);
  }
  // vysilani dalisho 'byte' pres preruseni
  void PaketSendUSARTNextByte(UART_HandleTypeDef *huart)
  {
    // pokud je jeste co vysilat
    if(paketSendLength > 0)
    {
        // data do vysilaciho registru
        huart->Instance->DR = *paketSendBuffer;

        paketSendBuffer++;  // posun ukazatele dat
        paketSendLength--;  // snizeni poctu jeste pro silani
    }
    // pokud jiz neni (ted / priste) co vysilat, tak zakazat preruseni
    if(paketSendLength == 0)
    {
        __HAL_UART_DISABLE_IT(huart, UART_IT_TXE);
        // vse odvysilano
        CommunFlags &= ~CommunSendRun;
    }
  }
#endif
