// komunikace pomoci paketu
// Petr Novak (novakpe@labe.felk.cvut.cz)
// 2016-01-11

#ifndef X_COMMUN_H_
#define X_COMMUN_H_

#include "XUtilsBase.h"

// *** nutno nastavit ***
#define PROC_32BIT          // 32bit procesor (ARM)
//#define PROC_8BIT         // 8bit procesor
//#define PAKET_ADDRESS     // 'Address' se pouzva v paketu
#define PAKET_IDENT       // 'Ident' se pouziva v paketu

//#define NULL 0

// *** konstanty ***

// zacatek a konec paketu
//#define PAKET_START '<'
//#define PAKET_END '>'

#define PAKET_START '<'
#define PAKET_END '>'
#define PAKET_END_TOTAL (char)0x00
#define GROUP_START '='			// zacatek skupiny (vice polozek)
#define GROUP_SEPARATOR ' '		// oddeleni skupin (vice polozek)
#define ITEMS_SEPARATOR ';'        // oddelovac polozek ve skupine
#define NUMBER_DEC_POINT '.'
#define NUMBER_DEC_COMMA ','

// *** definice ***

// pred-definovane 'PaketType' (char)
typedef enum _PaketType
{
    PaketTypeNone = (char)0x00, // vychozi typ paket (zadny uvodni znak)
    PaketTypeSend = (char)0x00, // vychozi typ paket (zadny uvodni znak)
    PaketTypeAns = '$',         // odpoved (uvodni znak '$')
} PaketTypes;

// pred-definovane 'PaketName' (char)
typedef enum _PaketName
{
    PaketNameNone = 0x00,           // nedefinovani / nepovoleno
    PaketNameCmd = 'C',             // 'Cmd' (povel)
    PaketNameData = 'D',            // 'Data' (datovy)
    PaketNameStatus = 'S',          // 'Status'
    PaketNameInfo = 'I'             // 'Info'
} PaketNames;

// pred-definovane povely (byte)
typedef enum _Cmds
{
    PaketCmdNone = 0,               // zadny povel
    PaketCmdReset = 1,              // RESET
    PaketCmdInfo = 2,               // ziskani info o zarizeni
    PaketCmdInit = 3,               // inicializace / otevreni zarizni (pokud je potreba)
    PaketCmdClose = 4,              // uzavreni / deinicializace zarizni (pokud je potreba)
    PaketCmdStop = 5,               // STOP / zastaveni
    PaketCmdStart = 6,              // START / spusteni
    PaketCmdCheck = 7,              // Check / kontrola stavu
    PaketCmdDevSet = 8,             // zakladni nastaveni zarizeni
    PaketCmdSetup = 9,              // s nastavenim
    PaketCmdUpdate = 10,            // aktualizace zarizeni

    PaketCmdSetParams = 11,         // parametry do zarizeni
    PaketCmdGetParams = 12,         // parametry ze zarizeni

    PaketCmdSetSpeed = 15,          // nastaveni (nejake) rychlosti
    PaketCmdSetGain = 16,           // nastaveni (nejakeho) zesileni
    PaketCmdSetData = 17,           // nastaveni (nejakych) dat
    PaketCmdGetValue = 19,          // cteni jedne sady dat

    PaketCmdDevSub = 21             // specificky pod-povel pro zarizeni
} PaketCmds;

// upresneni povelu pro 'Reset'
typedef enum
{
    ResetSubCmdHard = 1,            // HW 'Reset'
    ResetSubCmdSoft = 2,            // SW 'Reset'
} ResetSubCmds;

// upresneni povelu pro 'Info'
typedef enum
{
    InfoSubCmdDev = 1,              // Device 'Info'
    InfoSubCmdHW = 2,               // HW 'Info'
    InfoSubCmdSW = 3,               // SW 'Info'
} InfoSubCmds;

// upresneni povelu pro 'Setup'
typedef enum
{
    SetupSubCmdStore = 1,           // Setup -> EEProm
    SetupSubCmdLoad = 2,            // EEProm -> Setup
    SetupSubCmdClear = 3,           // Setup / EEProm -> Clear
} SetupSubCmds;

// pred-definovane 'Status' o vykonani povelu / paketu
typedef enum _Status
{
    PaketStatusNone = 0,
    PaketStatusOK = 1,
} PaketStatus;

// priznaky vyuzivane pri komunikaci
typedef enum _XCommunFlags
{
#ifdef PROC_32BIT
	CommunNone = 0x00000000,
	// prijem
	CommunRecRun = 0x00000001,		// prave se prijima paket
	//CommunRecHigh = 0x00000002,	// prijima se LOW / HIGH cast ASCII-HEX cisla
	CommunRecDone = 0x00000004,		// prijem paketu se (prave) dokoncil, paket je prijat
	// vysilani
	//Commu#ifndef X_COMMUN_H_
#define X_COMMUN_H_nSendHigh = 0x00000010,	// vysila se LOW / HIGH cast ASCII-HEX cisla
	CommunSendRun = 0x00000020,		// prave se vysila paket

	CommunSendAns = 0x00000080      // pozadavek na odeslani odpovedi
#endif
} XCommunFlags;

// *** pro externi pristup / vyuziti ***

#ifdef PROC_32BIT
  extern volatile uint32_t CommunFlags;
#endif

extern int paketRecLength;          // delka prijateho paketu (pokud je potreba)
#ifdef PAKET_ADDRESS
  extern int paketRecAddress;       // adresa obsazena v paketu (0 - neni)
#endif
extern char paketRecType;           // typ paketu
extern char paketRecName;           // nazev paketu
#ifdef PAKET_IDENT
  extern int paketRecIdent;         // ident paketu, pokud je pritomen (0 - neni)
#endif

extern byte cmdValue;  // cislo prijateho povelu
extern byte cmdStatus; // 'Status' o vykonani povelu
extern byte *paketSendBufferActual; // buffer pro prave vytvareny povel

// --- INIT (inicializace) ---

// inicializace 'Commun'
void CommunInit(byte *bufferIn, byte *bufferOut);

// --- RECEIVE (prijem) ---

// test na prijem paketu
void PaketRecvTestBufer(byte *bufferData, int bufferLength);
void PaketRecvTestByte(byte value);
byte PaketProcess(byte* paketOutBuffer);

// nalezeni pozadovaneho nazvu dat (ukazatel na zacatek techto dat)
byte* PaketRecvGetDataNamePos(char dataName);
// vyzvednuti prvni / jedne hodnoty za zadanych dat
int PaketRecvGetDataNumber(char dataName);
// vyzvednuti dalsi hodnoty ze jiz nastavenych dat
int PaketRecvGetDataNumberNext();
int PaketRectHasDataNumberNext();
// zda aktualne nalezena / nastavena data obsahuji pozadovany znak
byte IsCharInActualData(byte *paketPos, char oneChar);
// paket byl jiz zpracovan
void PaketRecvProcessed();

// --- SEND (vysilani) ---

// vytvoreni zacatku paketu s nazvem (vcetne START znaku)
void PaketSendBeginName(byte **paketPos, char paketName);
// vytvoreni zacatku paketu s typem a nazvem (vcetne START znaku)
void PaketSendBeginTypeName(byte **paketPos, char paketType, char paketName);
// vytvoreni konce paketu (vcetne STOP znaku)
// za posledni znak se ulozi '0x00' (pro jistotu)
// vrati delku paketu (pocet bytes)
int PaketSendEnd(byte *paketPosStart, byte **paketPos);

// zapis cisla z ukazatele do paketu jako posloupnost AsciiHex cislec
// (pokud je 'dataCount' zaporny -> cislo se zapisuje pozadu)
void PaketSendAddDataNameAndNumberHexPtr(byte **paketPos, char dataName, uint8_t *dataValuePtr, int dataCount);
// vlozeni jedne pojmenovane hodnoty do paketu (V=xxx)
void PaketSendAddDataNameAndNumber(byte **paketPos, char dataName, int dataValue);
// vlozeni pojmenovani pro pozdejsi hodnotu / hodnoty do paketu
void PaketSendAddDataName(byte **paketPos, char dataName);
// vlozeni dalsi hodnoty do paketu (za nastavenou pojmenovanou)
void PaketSendAddDataNumber(byte **paketPos, int dataValue);
// vlozeni nekolika hodnot s jednim pojmenovanim do paketu (V=xxx,yyy,...)
void PaketSendAddDataNameAndNumbers(byte **paketPos, char dataName, /* XValuesInBytes valueType, */ int dataCount, int* dataValues);
// vlozeni jednoho pojmenovaneho retezce
void PaketSendAddDataNameAndString(byte **paketPos, char dataName, const char* dataValues);
// sestaveni celeho jednoducheho paketu obsahujiciho jednu hodnotu
// (paket pouze sestaven, nikoli vyslan)
int PaketSendCreateSimple(byte *paketbuffer, char paketType, char paketName, char dataName, int dataValue);

#endif /* X_COMMUN_H */

