
#include "GlobalDef.h"
#include "eeprom.h"

// poznamky
// virtualni-adresa (16bit)
//  'High' cast (8bit) + poradove cislo polozky (8bit)

// konfigurace pro 'Setup'
extern SetupConfig setupConfig[];
extern uint16_t VirtAddVarTab[];

// vymazani celeho 'Setup' (vymazani stranek 'Flash')
void SetupClear(void)
{
    HAL_FLASH_Unlock();             // odemknuti FLASH
    HAL_StatusTypeDef status = EE_Format(); // vymazani FLASH
    HAL_FLASH_Lock();               // uzamknuti FLASH
}

// inicializace FLASH-EEPROM
// (vytvoreni tabulky virtualnich adresa)
// (inicializace FLASH pouzivane jako EEPROM)
void SetupInit(void)
{
    // inicializace / sestaveni tabulky pouzitych virtualnich adres
    int pos = 0, items, vaPos = 0;
    // projiti vsech 'Setup' (posledni ma 'VrtAdr=0')
    while(setupConfig[pos].adrHigh != 0)
    {
        // pocet polozek podle typu datovych polozek
        switch(setupConfig[pos].flags & SETUP_FLAG_TYPE_MASK)
        {
            case SETUP_FLAG_TYPE_8BIT: { items = setupConfig[pos].structSize / 1; break; }
            case SETUP_FLAG_TYPE_16BIT: { items = setupConfig[pos].structSize / 2; break; }
            //case SETUP_FLAG_TYPE_32BIT: { break; }
            //case SETUP_FLAG_TYPE_64BIT: { break; }
        }
        // v tabulce 'VrtAdr' bude tolik polozek, kolik je polozek v datech
        // (preskoceni prvnich a poslednich podle nastaveni)
        for(uint16_t item = setupConfig[pos].skipFirst; item < (items - setupConfig[pos].skipLast); item++)
        {
            // 'VrtAdr' pro jednu polozku = horni adresa + poradove cislo polozky
            // (vyrtualni adresy zacinaji od '1')
            VirtAddVarTab[vaPos++] = (uint16_t)((setupConfig[pos].adrHigh << 8) | (item + 1));
        }
        pos++;
    }
    // inicializace pro ukladani 'Setup'
    HAL_FLASH_Unlock();             // odemknuti FLASH
    HAL_StatusTypeDef status = EE_Init();   // inicializace FLASH
    HAL_FLASH_Lock();               // uzamknuti FLASH
}

// cteni dat z FLASH-EEPROM
void SetupLoad(void *loadData)
{
    int pos = 0, items;
    SetupConfig *stpCfgTemp;
    uint16_t dataLoad, vAdr;
    uint8_t *structData8bit; uint16_t *structData16bit;
    uint16_t status;
    HAL_FLASH_Unlock();
    // projiti vsech 'Setup' (posledni ma 'VrtAdr=0')
    while(setupConfig[pos].adrHigh != 0)
    {
        // pokud je predan 'SetupConfig' a neni to tento
        if ((loadData != NULL) && (setupConfig[pos].structData != loadData))
            // tak se tento preskoci
            { pos++; continue; }

        // pocet polozek a jejich typ podle typu datovych polozek
        switch(setupConfig[pos].flags & SETUP_FLAG_TYPE_MASK)
        {
            case SETUP_FLAG_TYPE_8BIT: {
                items = setupConfig[pos].structSize / 1;
                structData8bit = (uint8_t*)setupConfig[pos].structData;
                break; }
            case SETUP_FLAG_TYPE_16BIT: {
                items = setupConfig[pos].structSize / 2;
                structData16bit = (uint16_t*)setupConfig[pos].structData;
                break; }
            //case SETUP_FLAG_TYPE_32BIT: { break; }
            //case SETUP_FLAG_TYPE_64BIT: { break; }
        }
        // postupne projiti vsech datovyh polozek
        // (preskoceni prvnich a poslednich podle nastaveni)
        for(uint16_t item = setupConfig[pos].skipFirst; item < (items - setupConfig[pos].skipLast); item++)
        {
            // vytvoreni 'VrtAdr' pro tuto polozku
            vAdr = (uint16_t)((setupConfig[pos].adrHigh << 8) | (item + 1));
            // nacteni dat z FLASH
            status = EE_ReadVariable(vAdr, &dataLoad);
            // pokud bylo nacteni uspecne
            if (status == 0)
            {
                // zapis do dat podle typu polozky
                switch(setupConfig[pos].flags & SETUP_FLAG_TYPE_MASK)
                {
                    case SETUP_FLAG_TYPE_8BIT: { structData8bit[item] = (uint8_t)dataLoad; break; }
                    case SETUP_FLAG_TYPE_16BIT: { structData16bit[item] = (uint16_t)dataLoad; break; }
                }
            }
        }
        // neni zmena dat
        // (data nactena z FLASH jsou vychozi)
        setupConfig[pos].flags &= ~SETUP_FLAG_CHANGED;
        // posun na dalsi 'setupConfig'
        pos++;
    }
    // uzamknuti FLASH
    HAL_FLASH_Lock();
}

// zapis dat do FLASH-EEPROM
void SetupStore(void *storeData)
{
    int pos = 0, items;
    SetupConfig *stpCfgTemp;
    uint16_t dataStore, dataLoad, vAdr;
    uint8_t *structData8bit; uint16_t *structData16bit;
    uint16_t status, statusRead;
    // odemknuti FLASH
    HAL_FLASH_Unlock();
    // projiti vsech 'Setup' (posledni ma 'VrtAdr=0')
    while(setupConfig[pos].adrHigh != 0)
    {
        // pokud je predan 'SetupConfig' a neni to tento
        if ((storeData != NULL) && (setupConfig[pos].structData != storeData))
            // tak se tento preskoci
            { pos++; continue; }

        // pouze pokud je zmena data v tomto 'Setup'
        if ((setupConfig[pos].flags & SETUP_FLAG_CHANGED_MASK) != SETUP_FLAG_CHANGED)
            { pos++; continue; }
        // pocet polozek a jejich typ podle typu datovych polozek
        switch(setupConfig[pos].flags & SETUP_FLAG_TYPE_MASK)
        {
            case SETUP_FLAG_TYPE_8BIT: {
                items = setupConfig[pos].structSize / 1;
                structData8bit = (uint8_t*)setupConfig[pos].structData;
                break; }
            case SETUP_FLAG_TYPE_16BIT: {
                items = setupConfig[pos].structSize / 2;
                structData16bit = (uint16_t*)setupConfig[pos].structData;
                break; }
            //case SETUP_FLAG_TYPE_32BIT: { break; }
            //case SETUP_FLAG_TYPE_64BIT: { break; }
        }
        // postupne projiti vsech datocyh polozek
        // (preskoceni prvnich a poslednich podle nastaveni)
        for(uint16_t item = setupConfig[pos].skipFirst; item < (items - setupConfig[pos].skipLast); item++)
        {
            // vytvoreni 'VrtAdr' pro tuto polozku
            vAdr = (uint16_t)((setupConfig[pos].adrHigh << 8) | (item + 1));
            // vyzvednuti hodnoty polozky
            switch(setupConfig[pos].flags & SETUP_FLAG_TYPE_MASK)
            {
                case SETUP_FLAG_TYPE_8BIT: { dataStore = structData8bit[item]; break; }
                case SETUP_FLAG_TYPE_16BIT: { dataStore = structData16bit[item]; break; }
            }
            // vyzvednuti jiz ulozene hodnoty ve FLASH (puvodni)
            statusRead = EE_ReadVariable(vAdr, &dataLoad);
            // pokud hodnota ve FLASH neni
            // nebo je jina od aktualni v 'Setup'
            if ((statusRead != 0) || (dataLoad != dataStore))
            {
                // tak zapis polozky do FLASH (nova / zmena)
                status = EE_WriteVariable(vAdr, dataStore);
                //if (status != EE_OK)
                //    { int a = 100; } // test
            }
        }
        // pokud jsou data ze 'Setup' zapsana do FLASH
        // tak jsou v 'Setup' vychozi, jiz neni jejich zmena
        setupConfig[pos].flags &= ~SETUP_FLAG_CHANGED;
        // posun na dalsi 'setupConfig'
        pos++;
    }
    // uzamknuti FLASH
    HAL_FLASH_Lock();
}

// --- ---

/*
// nastaveni / nulovani priznaku zmeny dat
void SetChanged(void *structData, uint8_t flag)
{
    int pos = 0;
    SetupConfig *stpCfgTemp;
    // projiti vsech 'Setup' (posledni ma 'VrtAdr=0')
    while(setupConfig[pos].adrHigh != 0)
    {
        // ukazatel na jeden 'Setup'
        stpCfgTemp = &setupConfig[pos++];
        // nalezeni predavanych dat v 'Setup'
        // pokud to nejsou predavana data, tak se pokracuje dalsimi
        if (stpCfgTemp->structData != structData) { continue; }
        // vykonani akce na priznaku zmeny dat
        switch(flag)
        {
            // nulovani priznaku zmeny dat
            case SETUP_DATA_CHANGED_CLEAR: {
                stpCfgTemp->flags &= ~SETUP_FLAG_CHANGED;
                break; }
            // nastaveni priznaku zmeny dat
            case SETUP_DATA_CHANGED_SET: {
                stpCfgTemp->flags |= SETUP_FLAG_CHANGED;
                break; }
        }
        // zpracuji se vzdy pouze jedny data
        break;
    }
}
*/

// cteni / zapis hodnot nejakeho 'Report' <-> 'Setup'
// reportIn[0] = typ / cislo 'Setup'
void SetupToFromReport(uint8_t* reportIn, EnumReportSetupDir reportSetupDir)
{
    int pos = 0, items;
    SetupConfig *stpCfgTemp; // ukazatel na jeden 'Setup'
    uint8_t* structData;     // polozky v jednom 'Setup'
    // projiti vsech 'Setup' (posledni ma 'VrtAdr=0')
    while(setupConfig[pos].adrHigh != 0)
    {
        // odkaz na jeden 'Setup'
        stpCfgTemp = &setupConfig[pos];
        // pouze pokud jde o cislo zadaneho 'Setup'
        if (stpCfgTemp->adrHigh == *reportIn)
        {
            reportIn++; // posun na prvni hodnotu dat v 'Report'
            // pocet polozek typu 'uint8' v 'Setup'
            items = stpCfgTemp->structSize;
            // ukazatel na data bude typu 'uint8' (zatim)
            structData = (uint8_t*)stpCfgTemp->structData;
            // prepis vsech polozek (preskoceni prvnich a poslednich zadanych)
            for(uint8_t item = stpCfgTemp->skipFirst; item < (items - stpCfgTemp->skipLast); item++)
            {
                // 'Setup' -> 'Report'
                if (reportSetupDir == EnumSetupToReport)
                    // primy prepis hodnoty
                    { *reportIn = structData[item]; }
                // 'Report' -> 'Setup'
                if (reportSetupDir == EnumReportToSetup)
                {
                    // data se zapisi pouze pokud se lisi
                    // (kvuli nutnosti pozdejsimu zapisu do FLASH)
                    if(structData[item] != *reportIn)
                    {
                        // data se lisi, tak se ulozi nova hodnota
                        structData[item] = *reportIn;
                        // a je tedy zmena u tohoto 'Setup'
                        stpCfgTemp->flags |= SETUP_FLAG_CHANGED;
                    }
                }
                reportIn++; // posun na dalsi polozku v 'Report'
            }
            // hotovo, navrat
            break;
        }
        // posun na dalsi 'Setup'
        pos++;
    }
}

// --- ---
