
#include "GlobalDef.h"
#include "stm32l432xx.h"
#include "main.h"
#include "core_cm4.h"
#include "XUtils01.h"

const uint8_t bin2hex[16] = { (uint8_t)'0', (uint8_t)'1', (uint8_t)'2', (uint8_t)'3', (uint8_t)'4', (uint8_t)'5', (uint8_t)'6', (uint8_t)'7', (uint8_t)'8', (uint8_t)'9', (uint8_t)'A', (uint8_t)'B', (uint8_t)'C', (uint8_t)'D', (uint8_t)'E', (uint8_t)'F'};

// prevod 4bit cisla na jeden ASCII-HEX znak
inline uint8_t ByteToAsciiHex(uint8_t value)
{
    return (value <= (uint8_t)9) ? (value + (uint8_t)'0') : (value - (uint8_t)10 + (uint8_t)'A');
}

const uint32_t cnstResetToBoot = 0x11111111;
const uint32_t cnstResetToApp = 0x22222222;

#define X_SCS_BASE            (0xE000E000)                              /*!< System Control Space Base Address    */
#define X_SCB_BASE            (X_SCS_BASE +  0x0D00)                      /*!< System Control Block Base Address    */
#define X_SCB                 ((SCB_Type *) X_SCB_BASE)         /*!< SCB configuration struct             */

#define X_NVIC_AIRCR_VECTKEY    (0x5FA << 16)   /*!< AIRCR Key for write access   */
#define X_NVIC_SYSRESETREQ            2         /*!< System Reset Request         */

void XUTILS_SystemReset()
{
    // vykonani 'HW Reset'
  X_SCB->AIRCR  = (X_NVIC_AIRCR_VECTKEY | (X_SCB->AIRCR & (0x700)) | (1<<X_NVIC_SYSRESETREQ)); /* Keep priority group unchanged */
  __DSB();      /* Ensure completion of memory access */
  while(1);     /* wait until reset */
}

// vykonani 'HW Reset'
void XUTILS_Reset(EnumResetType resetType)
{

  /* This address within first 64k of memory: seemed to be problems with higher
   * locations. Note that this must match what's in startup.s
   */
  // nejaky 'Reset'
  *((unsigned long *)RESET_FLAGS_RAM_ADR) = 0x00000000;
  // do 'BootLoader'
  if(resetType == ResetTypeBoot)
    { *((unsigned long *)RESET_FLAGS_RAM_ADR) = cnstResetToBoot; }
  // do 'Application'
  if(resetType == ResetTypeApp)
    { *((unsigned long *)RESET_FLAGS_RAM_ADR) = cnstResetToApp; }
  // vykonani skutecneho 'Reset'
  XUTILS_SystemReset();
}



// --- ---
